package canvas

import (
	"hexasoftware/lib/vnode/vfunc"

	"dev.hexasoftware.com/stdio/wsrpc"
)

type Canvas struct {
	ctx         *wsrpc.ClientCtx
	OnResize    func(width, height int)
	OnMouseMove func(x, y int)
}

func New(ctx *wsrpc.ClientCtx) *Canvas {
	c := &Canvas{ctx, nil, nil}

	ctx.Define("OnResize", vfunc.NCastFunc(func(width, height float64) interface{} {
		if c.OnResize != nil {
			c.OnResize(int(width), int(height))
		}
		return nil
	}))
	ctx.Define("OnMouseMove", vfunc.NCastFunc(func(x, y float64) interface{} {
		if c.OnMouseMove != nil {
			c.OnMouseMove(int(x), int(y))
		}
		return nil
	}))

	return c
}

func (c *Canvas) Arc(x, y, radius, startAngle, endAngle int) {
	c.ctx.Call("DrawArc", x, y, radius, startAngle, endAngle)
}
func (c *Canvas) Line(x1, y1, x2, y2 int) {
	c.ctx.Call("Line", x1, y1, x2, y2)
}
func (c *Canvas) Clear() {
	c.ctx.Call("Clear")
}
