package main

import (
	"hexasoftware/lib/vnode"
	"testing"
)

func BenchmarkNative(b *testing.B) {
	sumFunc := func(a, b int) int {
		return a + b
	}
	ret10 := func() int {
		return 10
	}
	test := func() int {
		return sumFunc(sumFunc(1, ret10()), 2)
	}

	for i := 0; i < b.N; i++ {
		test()
	}

}

func BenchmarkNodeReflection(b *testing.B) {

	sumFunc := func(a, b int) int {
		return a + b
	}
	ret10 := func() int {
		return 10
	}

	main := vnode.New(sumFunc)
	vnode2 := vnode.New(sumFunc)
	vnode2.Source(ret10, 4)
	main.Source(1, vnode2)

	for i := 0; i < b.N; i++ {
		main.Exec()
	}

}

func BenchmarkNodeIface(b *testing.B) {
	sumFunc := func(p ...interface{}) interface{} {
		return p[0].(int) + p[1].(int)
	}
	ret10 := func() interface{} {
		return 10
	}

	main := vnode.New(sumFunc, 2)
	vnode2 := vnode.New(sumFunc, 2)
	vnode2.Source(ret10, 4)
	main.Source(1, vnode2)
	for i := 0; i < b.N; i++ {
		main.Exec()
	}

}
